@@ -26,7 +26,7 @@ use constant VALIDATE_HOSTNAME => eval 'require Data::Validate::Domain;1';
 use constant VALIDATE_IP       => eval 'require Data::Validate::IP;1';
 
 our $ERR;    # ugly hack to improve validation errors
-our $VERSION = '2.16';
+our $VERSION = '2.17';
 our @EXPORT_OK = qw(joi validate_json);
 
 my $BUNDLED_CACHE_DIR = path(path(__FILE__)->dirname, qw(Validator cache));
@@ -289,8 +289,8 @@ sub _load_schema_from_url {
 
   for (@{$self->cache_paths}) {
     my $path = path $_, $cache_file;
+    warn "[JSON::Validator] Looking for cached spec $path ($url)\n" if DEBUG;
     next unless -r $path;
-    warn "[JSON::Validator] Loading cached file $path\n" if DEBUG;
     return $self->_load_schema_from_text(\$path->slurp);
   }
 
@@ -1054,7 +1054,7 @@ JSON::Validator - Validate data against a JSON schema
 
 =head1 VERSION
 
-2.16
+2.17
 
 =head1 SYNOPSIS
 
